# QMSimFiles

[![Build Status](https://travis-ci.org/masuday/QMSimFiles.jl.svg?branch=master)](https://travis-ci.org/masuday/QMSimFiles.jl)

## Quick start

This package helps to manipulate data generated by [QMSim](https://animalbiosciences.uoguelph.ca/~msargol/qmsim/), a genome simulator.
This package can simulate progeny genotypes born from arbitrary parents, but it does not simulate the entire population.
The current version handles only genotypes; there is no way to have phenotypes, pedigrees, inbreeding coefficients.

## Examples

```julia
using QMSimFiles

# files simulated by QMSim
snpmap  = "lm_mrk_001.txt"
qtlmap  = "lm_qtl_001.txt"
qtleff  = "effect_qtl_001.txt"
snpdata = "p1_mrk_001.txt"
qtldata = "p1_qtl_001.txt"

# read a map file and genotypes at the same time
g = read_qmsim_data(snpmap, qtlmap, qtleff, snpdata, qtldata)

# chromosome map
map = g.map

# generate progeny genotypes by mating individuals 1 and 2
progeny = mating(map, g.individual[1], g.individual[2])

# save the entire data to HDF5
hdf5file = "population.h5"
save_qmsim_data_hdf5(g, hdf5file)

# add the new progeny to the file
add_qmsim_individual_hdf5(map, hdf5file, progeny)

# read the data from HDF5
updated_g = read_qmsim_data_hdf5(hdf5file)
```
